//  Copyright (c) 2004-2008 Ka-Ping Yee
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <stdio.h>
#include <math.h>
#include <time.h>

#define NV_PERIOD 1000000
#define NV_MAGICCONST 1.7155277699214135

#define N 624
#define M 397
#define MATRIX_A 0x9908b0dfUL   /* constant vector a */
#define UPPER_MASK 0x80000000UL /* most significant w-r bits */
#define LOWER_MASK 0x7fffffffUL /* least significant r bits */

typedef struct {
    unsigned long state[N];
    int index;
} randomstate;

void init_genrand(randomstate* state, unsigned long s) {
    int mti;
    unsigned long* mt = state->state;

    mt[0] = s & 0xffffffffUL;
    for (mti = 1; mti < N; mti++) {
            mt[mti] = 1812433253UL * (mt[mti-1] ^ (mt[mti-1] >> 30)) + mti;
            /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
            /* In the previous versions, MSBs of the seed affect   */
            /* only MSBs of the array mt[].                        */
            /* 2002/01/09 modified by Makoto Matsumoto             */
            mt[mti] &= 0xffffffffUL;
            /* for >32 bit machines */
    }
    state->index = mti;
    return;
}

unsigned long genrand_int32(randomstate* state) {
    unsigned long y;
    static unsigned long mag01[2]={0x0UL, MATRIX_A};
    /* mag01[x] = x * MATRIX_A  for x=0,1 */
    unsigned long* mt = state->state;

    if (state->index >= N) { /* generate N words at one time */
        int kk;

        for (kk=0; kk < N-M; kk++) {
            y = (mt[kk] & UPPER_MASK) | (mt[kk + 1] & LOWER_MASK);
            mt[kk] = mt[kk + M] ^ (y >> 1) ^ mag01[y & 0x1UL];
        }
        for (; kk < N-1; kk++) {
            y = (mt[kk] & UPPER_MASK) | (mt[kk + 1] & LOWER_MASK);
            mt[kk] = mt[kk + (M - N)] ^ (y >> 1) ^ mag01[y & 0x1UL];
        }
        y = (mt[N - 1] & UPPER_MASK) | (mt[0] & LOWER_MASK);
        mt[N - 1] = mt[M - 1] ^ (y >> 1) ^ mag01[y & 0x1UL];

        state->index = 0;
    }

    y = mt[state->index++];
    y ^= (y >> 11);
    y ^= (y << 7) & 0x9d2c5680UL;
    y ^= (y << 15) & 0xefc60000UL;
    y ^= (y >> 18);
    return y;
}

double randomdouble(randomstate* state) {
    unsigned long a = genrand_int32(state) >> 5;
    unsigned long b = genrand_int32(state) >> 6;
    return (double) (a*67108864.0 + b)*(1.0/9007199254740992.0);
}

double normalvariate(randomstate* state) {
    double u1, u2, z, zz;
    while (1) {
        u1 = randomdouble(state);
        u2 = 1.0 - randomdouble(state);
        z = NV_MAGICCONST * (u1 - 0.5) / u2;
        zz = z * z / 4.0;
        if (zz <= -log(u2)) break;
    }
    return z;
}

int main(int argc, char* argv[]) {
    FILE* fp;
    int i;
    double value;
    randomstate state;
    init_genrand(&state, 0);

    fp = fopen("random.dat", "w");
    for (i = 0; i < NV_PERIOD; i++) {
        value = normalvariate(&state);
        fwrite(&value, sizeof(value), 1, fp);
    }
    fclose(fp);
    return 0;
}
